@@ -10,37 +10,32 @@
 import numbers
 from struct import pack, unpack
 
-from blist import blist, btuple
+from recordtype import recordtype
 
-from python_patterns.utils.decorators import Property
-
-from .crypto import merkle
 from .mixins import HashableMixin, SerializableMixin
 from .numeric import mpq
 from .serialize import (
     serialize_varchar, deserialize_varchar,
     serialize_hash, deserialize_hash,
     serialize_list, deserialize_list)
-from .utils import StringIO, target_from_compact
+from .tools import StringIO, icmp, list, target_from_compact, tuple
 
 __all__ = [
     'ChainParameters',
     'Output',
-    'OutPoint',
     'Input',
     'Transaction',
-    'MerkleNode',
     'Block',
+    'ConnectedBlockInfo',
 ]
 
 # ===----------------------------------------------------------------------===
 
-from collections import namedtuple
 
-ChainParameters = namedtuple('ChainParameters', ['magic', 'port', 'genesis',
-    'testnet', 'max_value', 'transient_reward', 'transient_budget',
-    'perpetual_reward', 'perpetual_budget', 'fee_budget', 'maximum_target',
-    'next_target', 'alert_keys','checkpoints', 'features'])
+ChainParameters = recordtype('ChainParameters',
+    ['magic', 'port', 'genesis', 'testnet', 'max_value', 'transient_reward',
+     'transient_budget', 'perpetual_reward', 'perpetual_budget', 'fee_budget',
+     'maximum_target', 'next_target', 'alert_keys','checkpoints', 'features'])
 
 # ===----------------------------------------------------------------------===
 
@@ -72,8 +67,8 @@ class Output(SerializableMixin):
         return cls(**initargs)
 
     def __eq__(self, other):
-        return (self.amount   == other.amount and
-                self.contract == other.contract)
+        return all((self.amount   == other.amount,
+                    self.contract == other.contract))
     def __repr__(self):
         return '%s(amount=%d.%08d, contract=%s)' % (
             self.__class__.__name__,
@@ -83,98 +78,67 @@ class Output(SerializableMixin):
 
 # ===----------------------------------------------------------------------===
 
-class OutPoint(SerializableMixin):
-    def __init__(self, hash=0, n=0xffffffff, *args, **kwargs):
-        super(OutPoint, self).__init__(*args, **kwargs)
-        self.hash = hash
-        self.n = n
-
-    def serialize(self):
-        result  = serialize_hash(self.hash, 32)
-        result += pack('<I', self.n)
-        return result
-    @classmethod
-    def deserialize(cls, file_):
-        initargs = {}
-        initargs['hash'] = deserialize_hash(file_, 32)
-        initargs['n'] = unpack('<I', file_.read(4))[0]
-        return cls(**initargs)
-
-    def __nonzero__(self):
-        return not (self.hash==0 and self.n==0xffffffff)
-
-    def __eq__(self, other):
-        return self.hash==other.hash and self.n==other.n
-    def __repr__(self):
-        return '%s(hash=%064x, n=%d)' % (
-            self.__class__.__name__,
-            self.hash,
-            self.n==0xffffffff and -1 or self.n)
-
-# ===----------------------------------------------------------------------===
-
 class Input(SerializableMixin):
-    def __init__(self, outpoint=None, endorsement=None, sequence=0xffffffff,
-                 *args, **kwargs):
-        if outpoint is None:
-            outpoint = self.deserialize_outpoint(StringIO('\x00'*32 + '\xff'*4))
+    def __init__(self, output_hash=0, output_index=0xffffffff,
+                 endorsement=None, sequence=0xffffffff, *args, **kwargs):
         if endorsement is None:
             endorsement = kwargs.pop('coinbase', self.get_script_class()())
         super(Input, self).__init__(*args, **kwargs)
-        self.outpoint = outpoint
+        self.output_hash = output_hash
+        self.output_index = output_index
         self.endorsement = endorsement
         self.sequence = sequence
 
-    @classmethod
-    def get_outpoint_class(cls):
-        return getattr(cls, 'outpoint_class', OutPoint)
-
     @classmethod
     def get_script_class(cls):
         return getattr(cls, 'script_class', Script)
 
     def serialize(self):
-        result = self.outpoint.serialize()
+        result  = serialize_hash(self.output_hash, 32)
+        result += pack('<I', self.output_index)
         if hasattr(self.endorsement, 'serialize'):
             result += self.endorsement.serialize()
         else:
             result += serialize_varchar(self.endorsement) # <-- coinbase
         result += pack('<I', self.sequence)
         return result
-    @staticmethod
-    def deserialize_outpoint(file_):
-        return self.get_outpoint_class().deserialize(file_)
     @classmethod
     def deserialize(cls, file_):
         initargs = {}
-        initargs['outpoint'] = cls.deserialize_outpoint(file_)
+        initargs['output_hash'] = deserialize_hash(file_, 32)
+        initargs['output_index'] = unpack('<I', file_.read(4))[0]
         str_ = deserialize_varchar(file_) # <-- might be coinbase!
         initargs['sequence'] = unpack('<I', file_.read(4))[0]
-        if not initargs['outpoint'] and initargs['sequence']==0xffffffff:
+        if not all((initargs['output_hash']  == 0,
+                    initargs['output_index'] == 0xffffffff)):
             initargs['coinbase'] = str_
         else:
             initargs['endorsement'] = cls.get_script_class().deserialize(
                 StringIO(serialize_varchar(str_)))
         return cls(**initargs)
 
-    @Property
-    def is_coinbase():
-        def fget(self):
-            return not outpoint and sequence==0xffffffff
-        return locals()
+    @property
+    def is_coinbase(self):
+        return all((self.output_hash  == 0,
+                    self.output_index == 0xffffffff))
 
     def __eq__(self, other):
-        return (self.outpoint    == other.outpoint    and
-                self.endorsement == other.endorsement and
-                self.sequence    == other.sequence)
+        return all((self.output_hash  == other.output_hash,
+                    self.output_index == other.output_index,
+                    self.endorsement  == other.endorsement,
+                    self.sequence     == other.sequence))
+
     def __repr__(self):
         sequence_str = (self.sequence!=0xffffffff
             and ', sequence=%d' % self.sequence
              or '')
-        return '%s(outpoint=%s, %s=%s%s)' % (
+        return ('%s(output_hash=0x%064x, '
+                   'output_index=%d, '
+                   '%s=%s%s)') % (
             self.__class__.__name__,
-            repr(self.outpoint),
-            self.outpoint and 'endorsement' or 'coinbase',
+            self.output_hash,
+            self.output_index==0xffffffff and -1 or self.output_index,
+            self.is_coinbase and 'coinbase' or 'endorsement',
             repr(self.endorsement),
             sequence_str)
 
@@ -187,9 +151,9 @@ class Transaction(SerializableMixin, HashableMixin):
         if outputs is None: outputs = ()
         super(Transaction, self).__init__(*args, **kwargs)
         self.version = version
-        getattr(self, 'inputs_create', lambda x:setattr(x, 'inputs', blist()))(self)
+        getattr(self, 'inputs_create', lambda x:setattr(x, 'inputs', list()))(self)
         self.inputs.extend(inputs)
-        getattr(self, 'outputs_create', lambda x:setattr(x, 'outputs', blist()))(self)
+        getattr(self, 'outputs_create', lambda x:setattr(x, 'outputs', list()))(self)
         self.outputs.extend(outputs)
         self.lock_time = lock_time
         self.reference_height = reference_height
@@ -224,25 +188,23 @@ class Transaction(SerializableMixin, HashableMixin):
         initargs['version'] = unpack('<I', file_.read(4))[0]
         if initargs['version'] not in (1,2):
             raise NotImplementedError()
-        initargs['inputs'] = blist(deserialize_list(file_, lambda f:cls.deserialize_input(f)))
-        initargs['outputs'] = blist(deserialize_list(file_, lambda f:cls.deserialize_output(f)))
+        initargs['inputs'] = list(deserialize_list(file_, lambda f:cls.deserialize_input(f)))
+        initargs['outputs'] = list(deserialize_list(file_, lambda f:cls.deserialize_output(f)))
         initargs['lock_time'] = unpack('<I', file_.read(4))[0]
         if initargs['version'] in (2,):
             initargs['reference_height'] = unpack('<I', file_.read(4))[0]
         return cls(**initargs)
 
-    @Property
-    def is_coinbase():
-        def fget(self):
-            return 1==len(self.inputs) and not self.inputs[0].outpoint
-        return locals()
+    @property
+    def is_coinbase(self):
+        return 1==len(self.inputs) and not self.inputs[0].is_coinbase
 
     def __eq__(self, other):
-        return (self.version          == other.version          and
-                self.lock_time        == other.lock_time        and
-                self.reference_height == other.reference_height and
-                btuple(self.inputs)   == btuple(other.inputs)   and
-                btuple(self.outputs)  == btuple(other.outputs))
+        return all((self.version          == other.version,
+                    self.lock_time        == other.lock_time,
+                    self.reference_height == other.reference_height,
+                    icmp(iter(self.inputs),  iter(other.inputs))  == 0,
+                    icmp(iter(self.outputs), iter(other.outputs)) == 0))
     def __repr__(self):
         reference_height_str = (self.version in (2,)
             and ', reference_height=%d' % self.reference_height
@@ -260,113 +222,97 @@ class Transaction(SerializableMixin, HashableMixin):
 
 # ===----------------------------------------------------------------------===
 
-class MerkleNode(SerializableMixin, HashableMixin):
-    def __init__(self, children=None, *args, **kwargs):
-        if children is None: children = ()
-        super(MerkleNode, self).__init__(*args, **kwargs)
-        getattr(self, 'children_create', lambda x:setattr(x, 'children', blist()))(self)
-        for child in children:
-            if hasattr(child, 'hash') and not isinstance(child, MerkleNode):
-                child = child.hash
-            self.children.append(child)
-
-    # TODO: handle explicit merkle trees
-    def serialize(self):
-        if not all(map(lambda h:isinstance(h, numbers.Integral), self.children)):
-            raise NotImplementedError()
-        return serialize_list(self.children, lambda x:serialize_hash(x, 32))
-    @classmethod
-    def deserialize(cls, file_):
-        return cls(deserialize_list(file_, lambda x:deserialize_hash(x, 32)))
-
-    def hash__getter(self):
-        return merkle(self.children)
-
-    def __eq__(self, other):
-        return self.hash == other.hash
-    def __repr__(self):
-        return ''.join([
-            self.__class__.__name__,
-            '([',
-            ', '.join(map(repr, self.children)),
-            '])'])
-
-# ===----------------------------------------------------------------------===
+from .merkle import MerkleList
 
 class Block(SerializableMixin, HashableMixin):
-    def __init__(self, version=1, prev_block=0, merkle_root=None, time=0,
-                 bits=0x1d00ffff, nonce=0, vtx=None, *args, **kwargs):
-        if None not in (merkle_root, vtx):
-            if getattr(merkle_root, 'hash', merkle_root) != merkle(vtx):
-                raise ValueError(
-                    u"merkle_root does not match merkle(vtx); are you "
-                    u"sure you know what you're doing?")
-        else:
-            if vtx         is None: vtx         = ()
-            if merkle_root is None: merkle_root = MerkleNode(children=vtx)
+    def __init__(self, version=1, parent_hash=0, merkle_hash=0,
+                 time=0, bits=0x1d00ffff, nonce=0, *args, **kwargs):
+        parent = kwargs.pop('parent', None)
+        parent = getattr(parent, 'hash', parent)
+        parent_hash = getattr(parent_hash, 'hash', parent_hash)
+        if None not in (parent, parent_hash) and parent != parent_hash:
+            raise ValueError( u"parent.hash does not match parent_hash; are "
+                u"you sure you know what you're doing?")
+        parent_hash = parent_hash or parent or 0
+
+        merkle = kwargs.pop('merkle', None)
+        merkle = getattr(merkle, 'hash', merkle)
+        merkle_hash = getattr(merkle_hash, 'hash', merkle_hash)
+        if None not in (merkle, merkle_hash) and merkle != merkle_hash:
+            raise ValueError( u"merkle.hash does not match merkle_hash; are "
+                u"you sure you know what you're doing?")
+        merkle_hash = merkle_hash or merkle or 0
+
         super(Block, self).__init__(*args, **kwargs)
-        self.version = version
-        self.prev_block_hash = getattr(prev_block, 'hash', prev_block)
-        self.merkle_root_hash = getattr(merkle_root, 'hash', merkle_root)
-        self.time = time
-        self.bits = bits
-        self.nonce = nonce
+
+        self.version     = version
+        self.parent_hash = parent_hash
+        self.merkle_hash = merkle_hash
+        self.time        = time
+        self.bits        = bits
+        self.nonce       = nonce
 
     def serialize(self):
         if self.version not in (1,2):
             raise NotImplementedError()
         result  = pack('<I', self.version)
-        result += serialize_hash(self.prev_block_hash, 32)
-        result += serialize_hash(self.merkle_root_hash, 32)
+        result += serialize_hash(self.parent_hash, 32)
+        result += serialize_hash(self.merkle_hash, 32)
         result += pack('<I', self.time)
         result += pack('<I', self.bits)
         result += pack('<I', self.nonce)
         return result
     def __bytes__(self):
         return self.serialize()
-    @staticmethod
-    def deserialize_transaction(file_, *args, **kwargs):
-        return Transaction.deserialize(file_, *args, **kwargs)
     @classmethod
     def deserialize(cls, file_):
         initargs = {}
         initargs['version'] = unpack('<I', file_.read(4))[0]
         if initargs['version'] not in (1,2):
             raise NotImplementedError()
-        initargs['prev_block'] = deserialize_hash(file_, 32)
-        initargs['merkle_root'] = deserialize_hash(file_, 32)
+        initargs['parent_hash'] = deserialize_hash(file_, 32)
+        initargs['merkle_hash'] = deserialize_hash(file_, 32)
         initargs['time'] = unpack('<I', file_.read(4))[0]
         initargs['bits'] = unpack('<I', file_.read(4))[0]
         initargs['nonce'] = unpack('<I', file_.read(4))[0]
         return cls(**initargs)
 
     def __eq__(self, other):
-        return (self.version          == other.version          and
-                self.prev_block_hash  == other.prev_block_hash  and
-                self.merkle_root_hash == other.merkle_root_hash and
-                self.time             == other.time             and
-                self.bits             == other.bits             and
-                self.nonce            == other.nonce)
+        return all((self.version     == other.version,
+                    self.parent_hash == other.parent_hash,
+                    self.merkle_hash == other.merkle_hash,
+                    self.time        == other.time,
+                    self.bits        == other.bits,
+                    self.nonce       == other.nonce))
     def __repr__(self):
         return ('%s(version=%d, '
-                   'prev_block=0x%064x, '
-                   'merkle_root=0x%064x, '
+                   'parent_hash=0x%064x, '
+                   'merkle_hash=0x%064x, '
                    'time=%s, '
                    'bits=0x%08x, '
                    'nonce=0x%08x)' % (
             self.__class__.__name__,
             self.version,
-            self.prev_block_hash,
-            self.merkle_root_hash,
+            self.parent_hash,
+            self.merkle_hash,
             self.time,
             self.bits,
             self.nonce))
 
-    @Property
-    def difficulty():
-        def fget(self):
-            return mpq(2**256-1, target_from_compact(self.bits))
-        return locals()
+    @property
+    def difficulty(self):
+        return mpq(target_from_compact(0x1d00ffff),
+                   target_from_compact(self.bits))
+
+    @property
+    def work(self):
+        target = target_from_compact(self.bits)
+        if target < 0:
+            return 0
+        return (1<<256) // (target+1)
+
+ConnectedBlockInfo = recordtype('ConnectedBlockInfo',
+    ['parent', 'height', 'aggregate_work'])
 
 #
 # End of File
