@@ -8,18 +8,13 @@
 #
 
 import hashlib
-from itertools import count, izip
-import numbers
-
-from python_patterns.utils.decorators import Property
 
 from .serialize import serialize_hash, deserialize_hash
-from .utils import StringIO
+from .tools import StringIO
 
 __all__ = [
     'hash256',
     'hash160',
-    'merkle',
     'SECP256k1',
     'Secret',
     'InvalidSecretError',
@@ -68,32 +63,26 @@ class _NopHashAlgorithm(_HashAlgorithm):
         "Appends any passed in byte arrays to the “digest”."
         self._input = ''.join([self._input]+map(str,list(args)))
 
-    @Property
-    def block_size():
+    @property
+    def block_size(self):
         """`block_size` doesn't really have any meaning for the no-op/pass-
         through hash algorithm. Specifying `1` is effectively saying as
         much."""
-        def fget(self):
-            return 1
-        return locals()
+        return 1
 
-    @Property
-    def digest_size():
+    @property
+    def digest_size(self):
         """In a pass-through hash, the “digest size” is necessarily variable
         and not known until the input is finalized. Returning the current size
         of the input (which will become [a prefix of] the digest) is the best
         that we can do."""
-        def fget(self):
-            return len(self._input)
-        return locals()
+        return len(self._input)
 
-    @Property
-    def name():
+    @property
+    def name(self):
         """The defined canonical name of the the no-op/pass-through hash
         algorithm is the empty string `str('')`."""
-        def fget(self):
-            return ''
-        return locals()
+        return ''
 
 class _ChainedHashAlgorithm(_HashAlgorithm):
     """Implements a series of chained hash algorithms masquerading as single
@@ -146,30 +135,24 @@ class _ChainedHashAlgorithm(_HashAlgorithm):
             self._hobj.update(string)
         self._fobj = None
 
-    @Property
-    def block_size():
+    @property
+    def block_size(self):
         """Returns the block size (in bytes) of the hash algorithm. Computational
         cost and memory requirements are predictable if data is given to the hash
         object in unit increments of the block size."""
-        def fget(self):
-            return self._hobj.block_size
-        return locals()
+        return self._hobj.block_size
 
-    @Property
-    def digest_size():
+    @property
+    def digest_size(self):
         "Returns the size (in bytes) of the resulting digest."
-        def fget(self):
-            if getattr(self, '_fobj', None) is not None:
-                return self._fobj.digest_size
-            else:
-                return hashlib.new(self._algorithms[-1]).digest_size
-        return locals()
+        if getattr(self, '_fobj', None) is not None:
+            return self._fobj.digest_size
+        else:
+            return hashlib.new(self._algorithms[-1]).digest_size
 
-    @Property
-    def name():
-        def fget(self):
-            return self._algorithms
-        return locals()
+    @property
+    def name(self):
+        return self._algorithms
 
 class _HashAlgorithmInterface(tuple):
     """Provides a class API similar to `HASH` objects from `hashlib` (e.g.,
@@ -187,45 +170,36 @@ class _HashAlgorithmInterface(tuple):
         return hobj
     __call__=new
 
-    @Property
-    def digest_size():
+    @property
+    def digest_size(self):
         "Returns the size (in bytes) of the resulting digest."
-        def fget(self):
-            return self.new().digest_size
-        return locals()
+        return self.new().digest_size
 
-    @Property
-    def block_size():
+    @property
+    def block_size(self):
         """Returns the block size (in bytes) of the hash algorithm.
         Computational cost and memory requirements are predictable if data is
         given to the hash object in unit increments of the block size."""
-        def fget(self):
-            return self.new().block_size
-        return locals()
+        return self.new().block_size
 
-    @Property
-    def name():
+    @property
+    def name(self):
         "The defined canonical name of the the hash algorithm."
-        def fget(self):
-            return ':'.join(map(lambda n:n.encode('utf-8'), self))
-        return locals()
+        return ':'.join(map(lambda n:n.encode('utf-8'), self))
 
-hash256 = _HashAlgorithmInterface(('sha256', 'sha256'))
+    def serialize(self, hash_):
+        return serialize_hash(hash_, self.digest_size)
+    def deserialize(self, file_):
+        return deserialize_hash(file_, self.digest_size)
+
+# (sha256 . ripemd160) is used for Bitcoin addresses, as the 20-byte ripemd160
+# hash can save significant space.
 hash160 = _HashAlgorithmInterface(('sha256', 'ripemd160'))
 
-# ===----------------------------------------------------------------------===
-
-def merkle(hashes):
-    hashes = list(hashes)
-    for idx, child in filter(lambda h:not isinstance(h[1], numbers.Integral),
-                             izip(count(), hashes)):
-        hashes[idx] = merkle(child)
-    while len(hashes) > 1:
-        hashes = map(lambda h:serialize_hash(h, 32), hashes)
-        hashes.append(hashes[-1])
-        hashes = [deserialize_hash(StringIO(hash256(l+r).digest()), 32)
-                  for l,r in izip(*[iter(hashes)]*2)]
-    return hashes and hashes[0] or 0L
+# (sha256 . sha256), “double-SHA256” is used as the proof-of-work function,
+# the Merkle-tree compressor, and to generate transaction hash values for
+# identification.
+hash256 = _HashAlgorithmInterface(('sha256', 'sha256'))
 
 # ===----------------------------------------------------------------------===
 
