@@ -11,9 +11,7 @@ import six
 
 from struct import pack, unpack
 
-from .utils import StringIO
-
-from python_patterns.utils.decorators import Property
+from .tools import StringIO
 
 from .mixins import SerializableMixin
 from .serialize import (
@@ -467,20 +465,16 @@ class ScriptOp(SerializableMixin, six.binary_type):
 
         return super(ScriptOp, cls).__new__(cls, result, *args, **kwargs)
 
-    @Property
-    def opcode():
-        def fget(self):
-            return unpack('>B', self[:1])[0]
-        return locals()
+    @property
+    def opcode(self):
+        return unpack('>B', self[:1])[0]
 
-    @Property
-    def data():
-        def fget(self):
-            if self.opcode <= OP_PUSHDATA4:
-                return self[1:]
-            else:
-                return None
-        return locals()
+    @property
+    def data(self):
+        if self.opcode <= OP_PUSHDATA4:
+            return self[1:]
+        else:
+            return None
 
     def serialize(self):
         return self
@@ -510,75 +504,75 @@ class ScriptOp(SerializableMixin, six.binary_type):
             return cls(opcode, data)
         return cls(opcode)
 
-    @Property
-    def boolean():
-        def fget(self):
-            if self.opcode==OP_0:
-                return False
-            if self.opcode==OP_1NEGATE:
-                return True
-            if self.opcode>=OP_1 and self.opcode<=OP_16:
-                return True
-            if self.opcode in xrange(1,OP_PUSHDATA4+1):
-                return (
-                    any(map(lambda c:c!='\x00', self.data[:-1])) or
-                    self.data[-1] not in ('\x00', '\x80'))
-            else:
-                raise ValueError(u"non-data script-op cannot be interpreted as truth value")
-        def fset(self, value):
-            if value:
-                self.opcode, self.data = OP_TRUE, None
-            else:
-                self.opcode, self.data = OP_FALSE, None
-        return locals()
+    @property
+    def boolean(self):
+        if self.opcode==OP_0:
+            return False
+        if self.opcode==OP_1NEGATE:
+            return True
+        if self.opcode>=OP_1 and self.opcode<=OP_16:
+            return True
+        if self.opcode in xrange(1,OP_PUSHDATA4+1):
+            return (
+                any(map(lambda c:c!='\x00', self.data[:-1])) or
+                self.data[-1] not in ('\x00', '\x80'))
+        else:
+            raise ValueError(u"non-data script-op cannot be interpreted as truth value")
 
-    @Property
-    def integral():
-        def fget(self):
-            if self.opcode==OP_1NEGATE:
-                return -1
-            elif self.opcode==OP_0:
-                return 0
-            elif self.opcode>=OP_1 and self.opcode<=OP_16:
-                return self.opcode - OP_1 + 1
-            elif self.opcode in xrange(1,OP_PUSHDATA4+1):
-                data = self.data[:-1] + chr(ord(self.data[-1])&0x7f)
-                bignum = deserialize_hash(StringIO(data), len(data))
-                if ord(self.data[-1]) & 0x80:
-                    return -bignum
-                return bignum
+    @boolean.setter
+    def boolean(self, value):
+        if value:
+            self.opcode, self.data = OP_TRUE, None
+        else:
+            self.opcode, self.data = OP_FALSE, None
+
+    @property
+    def integral(self):
+        if self.opcode==OP_1NEGATE:
+            return -1
+        elif self.opcode==OP_0:
+            return 0
+        elif self.opcode>=OP_1 and self.opcode<=OP_16:
+            return self.opcode - OP_1 + 1
+        elif self.opcode in xrange(1,OP_PUSHDATA4+1):
+            data = self.data[:-1] + chr(ord(self.data[-1])&0x7f)
+            bignum = deserialize_hash(StringIO(data), len(data))
+            if ord(self.data[-1]) & 0x80:
+                return -bignum
+            return bignum
+        else:
+            raise ValueError(u"non-data script-op cannot be interpreted as integer")
+
+    @integral.setter
+    def integral(self, value):
+        self.data = None
+        if value == -1:
+            self.opcode = OP_1NEGATE
+        elif value == 0:
+            self.opcode = OP_0
+        elif value >= 1 and value <= 16:
+            self.opcode = OP_1 + value - 1
+        else:
+            neg = value < 1
+            absv = abs(value)
+            data = serialize_hash(absv, 1+len(bin(long(absv)).rstrip('L')[2:])//8)
+            if neg: data = data[:-1] + chr(ord(data[-1])|0x80)
+            datalen = len(data)
+            if datalen < OP_PUSHDATA1:
+                self.opcode = datalen
+            elif datalen <= 0xff:
+                self.opcode = OP_PUSHDATA1
+            elif datalen <= 0xffff:
+                self.opcode = OP_PUSHDATA2
+            elif datalen <= 0xffffffff:
+                self.opcode = OP_PUSHDATA4
             else:
-                raise ValueError(u"non-data script-op cannot be interpreted as integer")
-        def fset(self, value):
-            self.data = None
-            if value == -1:
-                self.opcode = OP_1NEGATE
-            elif value == 0:
-                self.opcode = OP_0
-            elif value >= 1 and value <= 16:
-                self.opcode = OP_1 + value - 1
-            else:
-                neg = value < 1
-                absv = abs(value)
-                data = serialize_hash(absv, 1+len(bin(long(absv)).rstrip('L')[2:])//8)
-                if neg: data = data[:-1] + chr(ord(data[-1])|0x80)
-                datalen = len(data)
-                if datalen < OP_PUSHDATA1:
-                    self.opcode = datalen
-                elif datalen <= 0xff:
-                    self.opcode = OP_PUSHDATA1
-                elif datalen <= 0xffff:
-                    self.opcode = OP_PUSHDATA2
-                elif datalen <= 0xffffffff:
-                    self.opcode = OP_PUSHDATA4
-                else:
-                    raise ValueError(u"integer representation exceeds serialization limits")
-                self.data = data
-        return locals()
+                raise ValueError(u"integer representation exceeds serialization limits")
+            self.data = data
 
     def __repr__(self):
         if self.opcode>0 and self.opcode<=OP_PUSHDATA4:
-            return ''.join(['0x', self.data.encode('hex')])
+            return ''.join(['\'', self.data.encode('hex'), '\'.decode(\'hex\')'])
         else:
             if self.opcode in OPCODE_NAMES:
                 return OPCODE_NAMES[self.opcode]
@@ -617,7 +611,7 @@ class Script(SerializableMixin, tuple):
 from .crypto import VerifyingKey
 from .defaults import CLIENT_VERSION
 from .serialize import SER_DISK, serialize_varint, deserialize_varchar
-from .utils import StringIO
+from .tools import StringIO
 
 class ScriptPickler(object):
     """Compact serializer for scripts.
@@ -730,9 +724,11 @@ class ScriptPickler(object):
     def get_script_class(self):
         return getattr(self, 'script_class', Script)
 
-    def dump(self, script):
+    def dump(self, script, file=None):
         "Write a compressed representation of script to the Pickler's file object."
-        self._dump(script, self._file, self._protocol, self._version)
+        if file is None:
+            file = self._file
+        self._dump(script, file, self._protocol, self._version)
 
     def dumps(self, script):
         "Return a compressed representation of script as a binary string."
@@ -740,10 +736,12 @@ class ScriptPickler(object):
         self._dump(script, string, self._protocol, self._version)
         return string.getvalue()
 
-    def load(self):
+    def load(self, file=None):
         "Read and decompress a compact script from the Pickler's file object."
+        if file is None:
+            file = self._file
         script_class = self.get_script_class()
-        script = self._load(self._file, self._protocol, self._version)
+        script = self._load(file, self._protocol, self._version)
         return script_class.deserialize(StringIO(script))
 
     def loads(self, string):
